unit FileOperations;

{ <WoW Addon-Updater>
  <Functions to Add/Move/Delete/Modify Files>

  Copyright (C) <2016> <Arndt Braier> <dev@braier.net>

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
}

{$mode objfpc}{$H+}

interface

uses
    Classes, SysUtils, Zipper;

procedure subZip(strZipAusgabeDatei: string; strListeZuPackenderDateien: TStringList);
procedure subUnZip(strZipDatei, strAusgabeOrdner: string);

function ExtractFilenameFromUrl(const AsURL : string) : string;
function CreateUniqueFileName(FileName: string): string;


implementation


// Procedures for Filenames (extract from URL, etc.)

function ExtractFilenameFromUrl(const AsURL : string) : string;
begin
  Result := trim(copy(AsUrl,LastDelimiter('/',AsUrl)+1,length(AsURL)));
end;


// Create unique filename

function CreateUniqueFileName(FileName: string): string;
var count: integer;
begin
  if not FileExists(FileName) then
  begin
    result := FileName;
    exit;
  end;
  count:=1;
  while FileExists(ChangeFileExt(FileName, '')+'.'+IntToStr(count)+
  ExtractFileExt(FileName)) do
    Inc(Count);
  result := ChangeFileExt(FileName,'')+'.'+IntToStr(count)+ExtractFileExt(FileName);
end;


// Procedures for ZIP and UnZIP

procedure subZip(strZipAusgabeDatei: string; strListeZuPackenderDateien: TStringList);
var
  OurZipper: TZipper;
begin
  OurZipper := TZipper.Create;
  try
    // Name der Zip-Datei, die erstellt werden soll
    OurZipper.FileName := strZipAusgabeDatei;
    // Stringliste mit den Dateien, die gepackt werden sollen
    OurZipper.Entries.AddFileEntries(strListeZuPackenderDateien);
    OurZipper.ZipAllFiles;
  finally
    FreeAndNil(OurZipper);
  end;
end;

procedure subUnZip(strZipDatei, strAusgabeOrdner: string);
var
  UnZipper: TUnZipper;
begin
  UnZipper := TUnZipper.Create;
  try
    // Name der Zip-Datei, die entpackt werden soll
    UnZipper.FileName := strZipDatei;
    // Name des Ziel-Verzeichnisses
    UnZipper.OutputPath := strAusgabeOrdner;
    UnZipper.Examine;
    UnZipper.UnZipAllFiles;
  finally
    FreeAndNil(UnZipper);
  end;
end;


end.

