{%mainunit uaddnew.pas}

{ <Include-File for uAddNew - Section Functions for Search and URL Input>
  <WoW Addon-Updater>
  
  Copyright (C) <2018-2019> <Arndt Braier> <dev@braier.net>
  
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
}

// Start Search when press Enter-Key
procedure TfrmAddNew.txtSearchKeyPress(Sender: TObject; var Key: char);
begin
  if Key = #13 then
    cmdSearch.Click;
end;


procedure TfrmAddNew.chkDirectURLChange(Sender: TObject);
begin
  frmAddNewReset; // Reset Form to Standard Values
  txtSearch.Text := '';

  if chkDirectURL.Checked = true then begin
    chkSearchEng.Visible := false;
    grpSearch.AnchorParallel(akRight, 5, Panel1);
    grpSearch.Caption := LangReqBoxAddonUrlText;
    RadioGroup1.Visible := false;
    Splitter1.Visible := false;
    grpResult.Visible := false;
  end
  else begin
    chkSearchEng.Visible := true;
    RadioGroup1.Visible := true;
    grpSearch.AnchorToNeighbour(akRight, 5, RadioGroup1);
    grpSearch.Caption := LangAddNewGrpSearch;
    grpResult.Visible := true;
    Splitter1.Visible := true;
  end;
  ActiveControl := txtSearch;
end;


procedure TfrmAddNew.cmdSearchClick(Sender: TObject);
var
  SearchResults: array of array[0..1] of string;
  SResult: IXQValue;
  S,
  STerm,
  SURL,
  SQuery,
  SHTML,
  SResultFilter,
  AddonName,
  AddonUrlPre,
  AddonUrlPost,
  AddonUrl: string;
  i: integer = 1;
  CurrRow: integer = 1;
  ItemsFound: integer;

begin
  if chkDirectURL.Checked = false then begin
    if (Length(txtSearch.Text) > 0) and (txtSearch.Text <> '##DEMO##') then begin
      frmAddNewReset; // Reset Form to Standard Values
      STerm := StringReplace(Trim(txtSearch.Text), ' ', '+', [rfReplaceAll]); // Format Search-Term

      // Step 1: Set variables
      if chkSearchEng.Checked = true then begin // Use Search-Engine
        case RadioGroup1.ItemIndex of
          0: begin // Curseforge
            SURL := 'https://duckduckgo.com/html/?keyword=' + STerm + '&q=site:curseforge.com/wow/addons/+' + STerm;
            SQuery := '//a[@class=''result__a'' and not(contains(., ''Files - '')) and not(contains(., ''Download - '')) and not(contains(., ''Changes - ''))]/concat(., '';;'', @href[not(contains(., ''?'')) and not(contains(., ''/download''))])';
            SResultFilter := 'curseforge.com/wow/addons/';
            AddonUrlPre := '';
            AddonUrlPost := '';
          end;
          1: begin // WoWInterface
            SURL := 'https://duckduckgo.com/html/?keyword=' + STerm + '&q=site:wowinterface.com/downloads/+' + STerm;
            SQuery := '//a[@class=''result__a'']/concat(., '';;'', @href[contains(., ''info'') and not(contains(., ''?''))])';
            SResultFilter := 'wowinterface.com/downloads/';
            AddonUrlPre := '';
            AddonUrlPost := '';
          end
        end
      end
      else if chkSearchEng.Checked = false then begin // Use Site-Search (Standard)
        //
        case RadioGroup1.ItemIndex of
          0: begin // Curseforge
            SURL := 'https://www.curseforge.com/wow/addons/search?search=' + STerm;
            // Routine changed with 3.1.0 due to changes on Curseforge website
            // Valid until 3.0.0: SQuery := '//div[@class=''list-item__details xs-mg-r-1'']/a/concat(., '';;'', @href)';
            SQuery := '//a[@class=''my-auto'']/concat(., '';;'', @href)';
            SResultFilter := '/wow/addons/';
            AddonUrlPre := 'https://www.curseforge.com';
            AddonUrlPost := '';
          end;
          1: begin // WoWInterface
            // ================================================================
            // CODE BACKUP FOR WOWINTERFACE PART 1/2:
            // Section needed only for first HTML retrieve, because of HTML/PHP
            // This can be solved with retrieve HTML two times, but leave code
            // here for backup purposes...
            {
            SURL := 'https://www.wowinterface.com/downloads/search.php?search=' + STerm;
            SQuery := '//a[@href[contains(., ''fileinfo'')]]/concat(., '';;'', @href)';
            SResultFilter := 'fileinfo.php';
            AddonUrlPre := 'https://www.wowinterface.com/downloads/info';
            AddonUrlPost := '.html';
            }
            // ================================================================
            SURL := 'https://www.wowinterface.com/downloads/search.php?search=' + STerm;
            SQuery := '//a[@class=''addonLink'']/concat(., '';;'', @href)';
            SResultFilter := 'info';
            AddonUrlPre := 'https://www.wowinterface.com/downloads/';
            AddonUrlPost := '';
            try
              retrieve(SURL); // First Download HTML of Search Results
            except
              On E :Exception do
                ShowMessage(E.Message);
            end
           end
        end
      end;

      // Step 2: Download HTML Search Result
      try
        SHTML := retrieve(SURL); // Download HTML of Search Results
      except
        On E :Exception do begin
          ShowMessage(E.Message);
          Exit
        end
      end;

      // Optimize HTML
      SHTML := StringReplace(SHTML, '/l/?kh=-1&amp;uddg=', '', [rfReplaceAll]);
      SHTML := StringReplace(SHTML, '%2D', '-', [rfReplaceAll]);
      SHTML := StringReplace(SHTML, '%2F', '/', [rfReplaceAll]);
      SHTML := StringReplace(SHTML, '%3A', ':', [rfReplaceAll]);
      SHTML := StringReplace(SHTML, '%3D', '=', [rfReplaceAll]);
      SHTML := StringReplace(SHTML, '%3F', '?', [rfReplaceAll]);

      // Step 3: Calculate needed Array-Size and set
      for SResult in process(SHTML, SQuery) do
      begin
        S := SResult.ToString;
        if Pos(SResultFilter, S) <> 0 then
          inc(i);
      end;
      SetLength(SearchResults, i);

      // Step 4: Now, filter the HTML-Content and fill array...
      S := '';
      i := 1;

      for SResult in process(SHTML, SQuery) do
      begin
        S := SResult.ToString;
        if Pos(SResultFilter, S) <> 0 then begin
          // Prepare Addon-URL
          // ================================================
          // CODE BACKUP FOR WOWINTERFACE PART 2/2:
          // Section needed only for WoWInterface, see above.
          // Leave code here for backup purposes...
          {
          if (chkSearchEng.Checked = false) and (RadioGroup1.ItemIndex = 1) then // for search on WoWInterface only
            AddonURL := Copy(S, (Pos('&id=', S)+4), Length(S))
          else // for all other search conditions
            AddonURL := Copy(S, (Pos(';;', S)+2), Length(S));
          }
          AddonURL := Copy(S, (Pos(';;', S)+2), Length(S));

          // Store Addon-URL in Array
          AddonURL := AddonUrlPre + Trim(AddonURL) + AddonUrlPost; // adding Prefix + Postfix to URL, if necessary
          SearchResults[i, 1] := AddonURL;

          // Prepare Addon-Name
          if chkSearchEng.Checked = false then // for Site-Search
            AddonName := Copy(S, 1, Pos(';;', S)-1)
          else if chkSearchEng.Checked = true then begin // for Search-Engine
            case RadioGroup1.ItemIndex of
              0: begin // Curseforge
                if Pos('...', S) <> 0 then
                  AddonName := Copy(S, 1, Pos('...', S)+2)
                else if Pos(' - Addons - ', S) <> 0 then
                  AddonName := Copy(S, 1, Pos(' - Addons - ', S))
                else
                  AddonName := Copy(S, 1, Pos(';;', S)-1);
              end;
              1: begin // WoWInterface
                AddonName := Copy(S, 1, Pos(' : ', S)-1);
              end
            end
          end;

          // Store Addon-Name in Array
          SearchResults[i, 0] := AddonName;
          inc(i);
        end
      end;

      // Step 5: Finally, write array to StringGrid
      try
        CurrRow := StringGridResult.RowCount;
        for CurrRow := CurrRow to High(SearchResults) do begin
          StringGridResult.Row := StringGridResult.RowCount;
          StringGridResult.RowCount := StringGridResult.RowCount + 1;
          StringGridResult.Cells[1, CurrRow] := '0';
          StringGridResult.Cells[2, CurrRow] := SearchResults[CurrRow, 0];
          StringGridResult.Cells[3, CurrRow] := SearchResults[CurrRow, 1];
        end;
        StringGridResult.Row := 0;
      finally
        SetLength(SearchResults, 0); // Free Memory of SearchResults
      end;

      if StringGridResult.RowCount > 0 then begin
        StringGridResult.Row := 1;
        ItemsFound := StringGridResult.RowCount - 1;
      end;

      StringGridResult.AutoSizeColumns;

      lblResults.Caption := IntToStr(SelEntries) + '/' + IntToStr(ItemsFound) + LangAddNewlblResults;
    end
    // BEGIN Demo-Mode (only for development)
    else if (Length(txtSearch.Text) > 0) and (txtSearch.Text = '##DEMO##') then begin
      frmAddNewReset; // Reset Form to Standard Values
      txtSearch.Text := '';
      CurrRow := StringGridResult.RowCount;
      for CurrRow := CurrRow to 40 do begin
        StringGridResult.Row := StringGridResult.RowCount;
        StringGridResult.RowCount := StringGridResult.RowCount + 1;
        StringGridResult.Cells[1, CurrRow] := '0';
        StringGridResult.Cells[2, CurrRow] := 'DEMO Addon-Entry No. ' + IntToStr(CurrRow);
        if (CurrRow mod 2 = 0) then
          StringGridResult.Cells[3, CurrRow] := 'https://www.wowinterface.com/downloads/info' + IntToStr(Random(10000)) + '.html'
        else
          StringGridResult.Cells[3, CurrRow] := 'https://www.curseforge.com/wow/addons/recount';
      end;
      StringGridResult.Row := 1;
      ItemsFound := StringGridResult.RowCount - 1;
      StringGridResult.AutoSizeColumns;
      lblResults.Caption := IntToStr(SelEntries) + '/' + IntToStr(ItemsFound) + LangAddNewlblResults;
    end
    // END Demo-Mode (only for development)
  end
  else if (chkDirectURL.Checked = true) and // Enter Direct URL
    (Pos('http', txtSearch.Text) <> 0) and
    (Pos('/', txtSearch.Text) <> 0) and
    (Pos(':', txtSearch.Text) <> 0) and
    (ListBox1.Items.IndexOf(txtSearch.Text) = -1) then // Check for existing entry
    begin
      frmAddNewReset; // Reset Form to Standard Values
      ListBox1.Items.Add(Trim(txtSearch.Text));
      txtSearch.Text := '';
      lblAddNew.Caption := IntToStr(ListBox1.Items.Count) + LangAddNewlblAddNew
    end
  else
    txtSearch.Color := clRed;
end;


procedure TfrmAddNew.RadioGroup1SelectionChanged(Sender: TObject);
begin
  case RadioGroup1.ItemIndex of
    0: begin // Curseforge
      frmAddNewSearchEnabled(true); // Turn Searchbox and Stringgrid on
      ActiveControl := txtSearch;
    end;
    1: begin // WoWInterface
      frmAddNewSearchEnabled(true); // Turn Searchbox and Stringgrid on
      ActiveControl := txtSearch;
    end;
    2: begin // ElvUI
      frmAddNewSearchEnabled(false); // Turn Searchbox and Stringgrid off
      if ListBox1.Items.IndexOf('ElvUI') = -1 then begin
        ListBox1.Items.Add('ElvUI');
        lblAddNew.Caption := IntToStr(ListBox1.Items.Count) + LangAddNewlblAddNew
      end;
    end;
    3: begin // Tukui
      frmAddNewSearchEnabled(false); // Turn Searchbox and Stringgrid off
      if ListBox1.Items.IndexOf('Tukui') = -1 then begin
        ListBox1.Items.Add('Tukui');
        lblAddNew.Caption := IntToStr(ListBox1.Items.Count) + LangAddNewlblAddNew
      end;
    end;
  end;
end;


procedure TfrmAddNew.cmdAddAllClick(Sender: TObject);
var
  CurrRow,
  TotalRows: integer;
  URL: string;

begin
  // Put selected Entries in ListBox
  with StringGridResult do begin
    if RowCount > 1 then begin
      TotalRows := RowCount - 1;
      for CurrRow := 1 to TotalRows do begin
        if Cells[1, CurrRow] = '1' then begin
          URL := Cells[3, CurrRow];
          if ListBox1.Items.IndexOf(URL) = -1 then // Check for existing entry
            ListBox1.Items.Add(URL);
        end;
      end;
      lblAddNew.Caption := IntToStr(ListBox1.Items.Count) + LangAddNewlblAddNew;
    end
  end;
  frmAddNewReset; // Reset Form to Standard Values
end;


procedure TfrmAddNew.ListBox1MouseUp(Sender: TObject; Button: TMouseButton;
    Shift: TShiftState; X, Y: Integer);
begin
  // Remove selected items
  if ListBox1.SelCount > 0 then begin
    if ListBox1.ItemIndex > -1 then
      ListBox1.Items.Delete(ListBox1.ItemIndex);

    if ListBox1.Items.Count > 0 then
      lblAddNew.Caption := IntToStr(ListBox1.Items.Count) + LangAddNewlblAddNew
    else
      lblAddNew.Caption := '';
  end;
end;


