{%mainunit umain.pas}

{ <Include-File for uMain - Section CheckForAddonUpdates>
  <WoW Addon-Updater>

  Copyright (C) <2016-2018> <Arndt Braier> <dev@braier.net>

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
}

constructor TChkAddUpdThread.Create(CreateSuspended : boolean);
begin
  FreeOnTerminate := True;
  inherited Create(CreateSuspended);
end;


procedure TChkAddUpdThread.StatusChkAddUpd;
// this method is executed by the mainthread and can therefore access all GUI elements.
begin
  if Assigned(FOnStatusChkAddUpd) then
  begin
    FOnStatusChkAddUpd(fCurrPos, fAddonStatusText, fAddonNameText, fAddonProgressText);
  end;
end;


procedure DoCheckForAddonUpdates;
begin
  with Form1 do begin
    // Disable Controls and Menue-Entries while Thread is running.
    cmdCheckForAddonUpdates.Enabled := False;
    cmdUpdateAddons.Enabled := False;
    cmdAddNewAddon.Enabled := False;
    cmdRemoveAddon.Enabled := False;
    cmdFilter.Enabled := False;
    mFile.Enabled := False;
    mEdit.Enabled := False;
    mAddons.Enabled := False;
    mUpdateAddons.Enabled := False;
    mSelectAddonDirectory.Enabled := False;
    mShowGnomeAnimations.Enabled := False;
    mSaveOptions.Enabled := False;

    LoadAddonList; // Load StringGrid to Array AddonTable
    StringGrid1.Enabled := false; // protect StringGrid for changes
    ProgressBar1.Visible := true;
    ProgressBar1.Max := StringGrid1.RowCount - 1;
    ProgressBar1.Position := 0;
    StatusBoxEnable; // Show StatusBox/Window

    // Create and start Thread
    ChkAddUpdThread := TChkAddUpdThread.Create(true);
    ChkAddUpdThread.OnStatusChkAddUpd := @StatusChkAddUpd;
    ChkAddUpdThread.Start;
    ChkAddUpdThread.OnTerminate := @OnDoneChkAddUpd;  // Start finishing procedure, if thread is done
  end;
end;


procedure TChkAddUpdThread.Execute; // Check Array for Addon Updates

// Explanation for Status-Column:
// ==========================================
// New     = New Addon or Update is available
// Updated = Addon was updated
// Ok      = No Update available (or needed)
// Error   = Something goes wrong              // Added with 2.1.0
//
// Explanation for Thread variables:
// fCurrPos           = Current Entry/Row in Array
// fAddonStatusText   = Text for Statusbox (What's doing now?)
// fAddonNameText     = Current Addon-Name
// fAddonProgressText = Progression in Percent

var
  CurrRow: integer = 1;
  StartTime: TDateTime;        // Save time of process start for time calculation
  RunTime: qword = 0;          // Process runtime for time calculation
  EstRunTime: qword = 0;       // Estimated runtime
  EstRunTimeText,              // Estimated runtime as formatted text
  PercentValue,                // Progress percentage as formatted text
  AddonUrl,
  AddonName,
  AddonVersion: string;        // Requested Version from Website
  i: integer;

begin
  try
    ChangesCounter := 0;  // Reset the ChangesCounter
    StartTime := Now;     // Save time of process start for time calculation
    EstRunTimeText := ''; // Initialize

    // Create Logfile entry
    ActLogNewEntry('[' + LangMnuChkAddUpd + ']', LangStatActStart);

    // At first, replace the "Updated" status entries from table (only for selected Addons)
    for CurrRow := 1 to High(AddonTable) do begin
      if (AddonTable[CurrRow, 1] = '1') AND (AddonTable[CurrRow, 7] = 'Updated') then
         AddonTable[CurrRow, 7] := 'Ok';
    end;

    // Now we look for Updates (only for selected Addons)
    for CurrRow := 1 to High(AddonTable) do begin
      if AddonTable[CurrRow, 1] = '1' then begin
        // Check Addon-Version from Website and compare it with list entry
        AddonUrl := AddonTable[CurrRow, 2];
        AddonName := AddonTable[CurrRow, 3];

        // Calculation of progress and estimated runtime
        PercentValue := FloatToStrF((CurrRow / (High(AddonTable) * 0.01)), ffFixed, 3, 0) + ' %';

        If CurrRow > 1 then begin
          RunTime :=  MillisecondsBetween(StartTime,Now);
          EstRunTime := (RunTime div (CurrRow -1)) * ((High(AddonTable) - CurrRow) +1);
          if EstRunTime > 60000 then
            EstRunTimeText := '   (' + IntToStr(EstRunTime div 60000) + ' min, '
            + IntToStr((EstRunTime div 1000) - ((EstRunTime div 60000) * 60)) + ' s)'
          else
            EstRunTimeText := '   (' + IntToStr(Round(EstRunTime/1000)) + ' s)';
        end;

        // Update Status-Messages and Logfile
        fCurrPos := CurrRow;
        fAddonStatusText := LangStatCheckUpdate;
        fAddonNameText := AddonName;
        fAddonProgressText := PercentValue + EstRunTimeText;
        ActLogNewEntry('  [' + fAddonStatusText + ']', '#' + IntToStr(fCurrPos)
        + ': ' + fAddonNameText + ' (' + fAddonProgressText + ').');
        Synchronize(@StatusChkAddUpd);

        // sleep(300); // Wait - only for testing

        AddonVersion := GetAddonVersion(AddonUrl);

        if AddonTable[CurrRow, 4] <> AddonVersion then begin
          inc(ChangesCounter);

          // Create Logfile-Entry
          ActLogNewEntry('    ' + LangStatLogNewAddVer + AddonVersion,
          '(' + LangStatWas + ': ' + AddonTable[CurrRow, 4] + ')');

          AddonTable[CurrRow, 4] := AddonVersion;
          AddonTable[CurrRow, 5] := GetAddonURL(AddonUrl);

          // Create Logfile-Entry
          ActLogNewEntry('    Download-URL: ', AddonTable[CurrRow, 5]);

          AddonTable[CurrRow, 7] := 'New';
        end;

        // Error-Handling
        For i:=1 To 6 Do begin
          if AddonTable[CurrRow, i] = 'n/a' then
            AddonTable[CurrRow, 7] := 'Error';
        end;

      end;
    end;
  except
    MessageDlg('Error', 'Error in Procedure while checking Array for Addon Updates.', mtError, [mbOK], 0);
  end;
end;


procedure TForm1.StatusChkAddUpd(CurrPosValue: integer; AddonStatusValue, AddonNameValue, AddonProgressValue: String);
// Explanation for Thread variables:
// CurrPosValue       = Current Entry/Row in Array
// AddonStatusValue   = Text for Statusbox (What's doing now?)
// AddonNameValue     = Current Addon-Name
// AddonProgressValue = Progression in Percent
begin
  StatusBar1.Panels[0].Text := AddonStatusValue + AddonNameValue;

  {$IFDEF DARWIN} // Compiler-Switch for OS X
  with Form1 do begin
  {$ELSE}
  with frmStatus do begin
  {$ENDIF}
    lblStatus.Caption := AddonStatusValue;
    lblAddonName.Caption := '#' + IntToStr(CurrPosValue) + ': ' + AddonNameValue;
    lblProgress.Caption := AddonProgressValue;
  end;

  ProgressBar1.Position := CurrPosValue;
end;


procedure TForm1.OnDoneChkAddUpd(ASender : TObject);
begin
  StringGrid1.Enabled := true; // make StringGrid writeable
  SaveAddonList;  // Put Array AddonTable back to StringGrid
  RestoreSelections; // Restore user-defined selections in Addon list
  StringGrid1.Row := 0;
  StringGrid1.Refresh;
  ProgressBar1.Visible := false;
  ProgressBar1.Position := 0;
  CalcListItems;
  SortAddonList('State'); // Sort function accepts 3 parameters: 'Number', 'Name' and 'State'

  {$IFDEF DARWIN} // Compiler-Switch for OS X
  with Form1 do begin
  {$ELSE}
  with frmStatus do begin
  {$ENDIF}
    lblStatus.Caption := LangStatFinished;
    lblAddonName.Caption := IntToStr(ChangesCounter) + LangStatCheckUpdateDone;
    lblProgress.Caption := LangAddonsTotal + IntToStr(NumberOfItems);
  end;

  // Create Logfile-Entry
  ActLogNewEntry('  [' + LangStatCheckUpdate + LangStatFinished + ']', IntToStr(ChangesCounter)
  + LangStatCheckUpdateDone + ' ' + LangAddonsTotal + IntToStr(NumberOfItems));
  ActLogNewEntry('[' + LangMnuChkAddUpd + ']', LangStatActEnd);
  ActLogSeparator; // Create separator line in logfile

  TimerStatusClose.Enabled := true; // Delay-Timer for StatusBox
  StatusBar1.Panels[0].Text := IntToStr(ChangesCounter) + LangStatCheckUpdateDone + ' ' + LangAddonsTotal + IntToStr(NumberOfItems);

  CheckedForAddonUpdates := true; // After update-check the cmdUpdateAddons-Button will be enabled
  ChangesCounter := 0; // Reset the ChangesCounter
  if AllowSortAddonList = false then
     StringGrid1.Row := SelMark;
  AllowSortAddonList := true;
end;


procedure DoCheckForUpdateSingleAddon;
begin
  with Form1 do begin
    SelMark := StringGrid1.Row; // Remember selected row
    AllowSortAddonList := false; // Switch off AddonList sort temporary
    SaveSelections; // Save user-defined selections in Addon list
    SelectNone;
    StringGrid1.Cells[1, SelMark] := '1';
    DoCheckForAddonUpdates;
  end;
end;


