{%mainunit umain.pas}

{ <Include-File for uMain - Section DoUpdateAddons>
  <WoW Addon-Updater>

  Copyright (C) <2016> <Arndt Braier> <dev@braier.net>

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
}

constructor TDoAddUpdThread.Create(CreateSuspended : boolean);
begin
  FreeOnTerminate := True;
  inherited Create(CreateSuspended);
end;


procedure TDoAddUpdThread.StatusDoAddUpd;
// this method is executed by the mainthread and can therefore access all GUI elements.
begin
  if Assigned(FOnStatusDoAddUpd) then
  begin
    FOnStatusDoAddUpd(fCurrPos, fAddonStatusText, fAddonNameText, fAddonProgressText);
  end;
end;


procedure DoUpdateAddons;
begin
  with Form1 do begin
    // Disable Controls and Menue-Entries while Thread is running.
    cmdCheckForAddonUpdates.Enabled := False;
    cmdUpdateAddons.Enabled := False;
    cmdAddNewAddon.Enabled := False;
    cmdRemoveAddon.Enabled := False;
    cmdFilter.Enabled := False;
    mFile.Enabled := False;
    mEdit.Enabled := False;
    mAddons.Enabled := False;
    mUpdateAddons.Enabled := False;
    mSelectAddonDirectory.Enabled := False;
    mShowGnomeAnimations.Enabled := False;
    mSaveOptions.Enabled := False;

    NumberOfNewSelItems := 0;
    NumberOfNewSelItems := CountNewSelectedAddons(StringGrid1);

    LoadAddonList; // Load StringGrid to Array AddonTable
    StringGrid1.Enabled := false; // protect StringGrid for changes
    ProgressBar1.Visible := true;
    ProgressBar1.Max := NumberOfNewSelItems;
    ProgressBar1.Position := 0;
    StatusBoxEnable; // Show StatusBox/Window

    // Create and start Thread
    DoAddUpdThread := TDoAddUpdThread.Create(true);
    DoAddUpdThread.OnStatusDoAddUpd := @StatusDoAddUpd;
    DoAddUpdThread.Start;
    DoAddUpdThread.OnTerminate := @OnDoneDoAddUpd;  // Start finishing procedure, if thread is done
  end;
end;


procedure TDoAddUpdThread.Execute; // Use Array to update Addons

// Explanation for Status-Column:
// ==========================================
// New     = New Addon or Update is available
// Updated = Addon was updated
// Ok      = No Update available (or needed)
// Error   = Something goes wrong              // Added with 2.1.0
//
// Explanation for Thread variables:
// fCurrPos           = Current Entry/Row in Array
// fAddonStatusText   = Text for Statusbox (What's doing now?)
// fAddonNameText     = Current Addon-Name
// fAddonProgressText = Progression in Percent

var
  CurrRow: integer = 1;
  AddonUrl,
  AddonName: string;

begin
  try
    ChangesCounter := 0; // Reset the ChangesCounter

    // Create Logfile entry
    ActLogNewEntry('[' + LangMnuUpdAdd + ']', LangStatActStart);

    // Now we start the update process along two categories:
    // Selection must be '1' and State 'New", otherwise do nothing
    for CurrRow := 1 to High(AddonTable) do begin
      if (AddonTable[CurrRow, 1] = '1') and (AddonTable[CurrRow, 7] = 'New') then begin // Check, if selection is true and Addon needs update
        inc(ChangesCounter);
        AddonUrl := AddonTable[CurrRow, 5];
        AddonName := AddonTable[CurrRow, 3];

        // Update Status-Messages and Logfile
        fCurrPos := ChangesCounter;
        fAddonStatusText := LangStatUpdateAddon;
        fAddonNameText := AddonName;
        fAddonProgressText := FloatToStrF((ChangesCounter / (NumberOfNewSelItems * 0.01)), ffFixed, 3, 0) + ' %';
        ActLogNewEntry('  [' + fAddonStatusText + ']', '#' + IntToStr(fCurrPos)
        + ': ' + fAddonNameText + ' (' + fAddonProgressText + ').');
        Synchronize(@StatusDoAddUpd);

        // sleep(300); // Wait - only for testing

        // Start Update process and update Addon table
        if UpdateAddon(AddonUrl, AddonDir) = True then
          AddonTable[CurrRow, 7] := 'Updated';

        // Create Logfile-Entry
        ActLogNewEntry('    Download-URL: ', AddonUrl);
        ActLogNewEntry('    ' + LangStatLogNewAddVer, AddonTable[CurrRow, 4]);
      end;
    end;
  except
    MessageDlg('Error', 'Error in Procedure while use Array to update Addons.', mtError, [mbOK], 0);
  end;
end;


procedure TForm1.StatusDoAddUpd(CurrPosValue: integer; AddonStatusValue, AddonNameValue, AddonProgressValue: String);
// Explanation for Thread variables:
// CurrPosValue       = Current Entry/Row in Array
// AddonStatusValue   = Text for Statusbox (What's doing now?)
// AddonNameValue     = Current Addon-Name
// AddonProgressValue = Progression in Percent
begin
  StatusBar1.Panels[0].Text := AddonStatusValue + AddonNameValue;

  {$IFDEF DARWIN} // Compiler-Switch for OS X
  with Form1 do begin
  {$ELSE}
  with frmStatus do begin
  {$ENDIF}
    lblStatus.Caption := AddonStatusValue;
    lblAddonName.Caption := '#' + IntToStr(CurrPosValue) + ': ' + AddonNameValue;
    lblProgress.Caption := AddonProgressValue;
  end;

  ProgressBar1.Position := CurrPosValue;
end;


procedure TForm1.OnDoneDoAddUpd(ASender : TObject);
begin
  StringGrid1.Enabled := true; // make StringGrid writeable
  SaveAddonList;  // Put Array AddonTable back to StringGrid
  RestoreSelections; // Restore user-defined selections in Addon list
  StringGrid1.Row := 0;
  StringGrid1.Refresh;
  ProgressBar1.Visible := false;
  ProgressBar1.Position := 0;
  CalcListItems;
//  SortAddonList('State'); // Sort function accepts 3 parameters: 'Number', 'Name' and 'State'

  {$IFDEF DARWIN} // Compiler-Switch for OS X
  with Form1 do begin
  {$ELSE}
  with frmStatus do begin
  {$ENDIF}
    lblStatus.Caption := LangStatFinished;
    lblAddonName.Caption := IntToStr(ChangesCounter) + LangStatUpdateAddonDone;
    lblProgress.Caption := LangAddonsTotal + IntToStr(NumberOfItems);
  end;

  // Create Logfile-Entry
  ActLogNewEntry('  [' + LangStatFinished + ']', IntToStr(ChangesCounter)
  + LangStatUpdateAddonDone + ' ' + LangAddonsTotal + IntToStr(NumberOfItems));
  ActLogNewEntry('[' + LangMnuUpdAdd + ']', LangStatActEnd);
  ActLogSeparator; // Create separator line in logfile

  TimerStatusClose.Enabled := true; // Delay-Timer for StatusBox
  StatusBar1.Panels[0].Text := IntToStr(ChangesCounter) + LangStatUpdateAddonDone + ' ' + LangAddonsTotal + IntToStr(NumberOfItems);

  CheckedForAddonUpdates := false; // After update-check the cmdUpdateAddons-Button will be enabled
  NumberOfNewSelItems := 0; // Reset NumberOfNewSelItems
  ChangesCounter := 0; // Reset the ChangesCounter
  if AllowSortAddonList = false then
     StringGrid1.Row := SelMark;
  AllowSortAddonList := true;
end;


procedure DoUpdateSingleAddon;
begin
  with Form1 do begin
    DoCheckForUpdateSingleAddon; // First, look for an update
    ChkAddUpdThread.WaitFor; // Waiting for Update Thread

    if StringGrid1.Cells[7, SelMark] <> 'Error' then
      StringGrid1.Cells[7, SelMark] := 'New'; // Set "New" flag to force update

    AllowSortAddonList := false; // Switch off AddonList sort temporary
    SaveSelections; // Save user-defined selections in Addon list
    SelectNone;
    StringGrid1.Cells[1, SelMark] := '1';
    DoUpdateAddons; // Finally Update Addon
  end;
end;


