{%mainunit umain.pas}

{ <Include-File for uMain - Section DoAddNewAddon>
  <WoW Addon-Updater>

  Copyright (C) <2016-2018> <Arndt Braier> <dev@braier.net>

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
}

procedure DoAddNewAddon;

// Explanation for Status-Column:
// ==========================================
// New     = New Addon or Update is available
// Updated = Addon was updated
// Ok      = No Update available (or needed)
// Error   = Something goes wrong              // Added with 2.1.0

const
  AddonUrlElv: string = 'https://git.tukui.org/elvui/elvui';
  AddonUrlTuk: string = 'https://git.tukui.org/Tukz/Tukui';

var
    AddonType: char;
    CurrRow: integer;
    AddonUrl: string;
    i, n: integer;

begin

with Form1 do
begin
    // Now we would add a new Addon-Entry.
    // At first, turn on Progressbar
    ProgressBar1.Visible := true;
    ProgressBar1.Max := 5;
    ProgressBar1.Position := 1;

    // Request Addon-URL from User
    StatusBar1.Panels[0].Text := LangStatAddStepA;

    // Startvalues
    AddonUrlList := TStringList.Create;
    AddonUrl := '';
    AddNewResult := true;

    // Open new Search and Input Dialog (implemented with 3.0)
    frmAddNew.Left := Left + ((Width - frmAddNew.Width) DIV 2);
    frmAddNew.Top := Top + ((Height - frmAddNew.Height) DIV 2);
    frmAddNew.ShowModal;

    if AddNewResult = false then begin // if user choose to cancel it
      StatusBar1.Panels[0].Text := LangStatStart + IntToStr(NumberOfItems);
      AddonUrlList.Free;
      ProgressBar1.Visible := false;
      exit
    end;

    // Processing AddonUrlList...
    if AddonUrlList.Count <> 0 then begin
      for i := 0 to AddonUrlList.Count -1 do begin
        ProgressBar1.Position := 1;

        // Handover AddonUrl
        AddonUrl := AddonUrlList[i];

        // Set AddonUrl for ElvUI + Tukui
        case LowerCase(AddonUrl) of
          'elvui': AddonUrl := AddonUrlElv;
          'tukui': AddonUrl := AddonUrlTuk;
        end;

        // Jump to end of table and add a new row.
        StringGrid1.Row := StringGrid1.RowCount;
        StringGrid1.RowCount := StringGrid1.RowCount + 1;
        CurrRow := StringGrid1.RowCount-1;
        StringGrid1.Cells[0, CurrRow] := IntToStr(CurrRow);

        // Switch-Update checkbox to true
        StringGrid1.Cells[1, CurrRow] := '1';

        // Put Addon-URL in Column
        StringGrid1.Cells[2, CurrRow] := AddonUrl;

        // Request Addon Name
        StatusBar1.Panels[0].Text := LangStatAddStepB;
        Application.ProcessMessages;
        StringGrid1.Cells[3, CurrRow] := GetAddonName(AddonUrl);
        Application.ProcessMessages;
        ProgressBar1.Position := ProgressBar1.Position + 1;

        // Request Addon Version
        StatusBar1.Panels[0].Text := LangStatAddStepC;
        Application.ProcessMessages;
        StringGrid1.Cells[4, CurrRow] := GetAddonVersion(AddonUrl);
        Application.ProcessMessages;
        ProgressBar1.Position := ProgressBar1.Position + 1;

        // Request Addon File URL
        StatusBar1.Panels[0].Text := LangStatAddStepD;
        Application.ProcessMessages;
        StringGrid1.Cells[5, CurrRow] := GetAddonURL(AddonUrl);
        Application.ProcessMessages;
        ProgressBar1.Position := ProgressBar1.Position + 1;

        // Add Addon-AddonSource to column
        Application.ProcessMessages;

        AddonType := GetAddonType(AddonUrl);

        case AddonType of
          'E': StringGrid1.Cells[6, CurrRow] := 'ElvUI';        // ElvUI
          'T': StringGrid1.Cells[6, CurrRow] := 'Tukui';        // Tukui
          'W': StringGrid1.Cells[6, CurrRow] := 'WoWInterface'; // WoWInterface
          'C': StringGrid1.Cells[6, CurrRow] := 'Curseforge';   // Curseforge
          '-': StringGrid1.Cells[6, CurrRow] := 'n/a';          // Error / nothing found
        end;

        // Set Status to New
        StringGrid1.Cells[7, CurrRow] := 'New'; // Values are: New, Updated, Ok
        Application.ProcessMessages;
        ProgressBar1.Position := ProgressBar1.Position + 1;
        CalcListItems;
        StatusBar1.Panels[0].Text := StringGrid1.Cells[3, CurrRow] + LangStatAddFinish + ' ' + LangAddonsTotal + IntToStr(NumberOfItems);

        // Create Logfile entries
        ActLogNewEntry('[' + LangMnuAddAddon + ']', LangStatActStart);
        ActLogNewEntry('  ' + LangStatAddStepA, StringGrid1.Cells[2, CurrRow]);
        ActLogNewEntry('  ' + LangStatAddStepB, StringGrid1.Cells[3, CurrRow]);
        ActLogNewEntry('  ' + LangStatAddStepC, StringGrid1.Cells[4, CurrRow]);
        ActLogNewEntry('  ' + LangStatAddStepD, StringGrid1.Cells[5, CurrRow]);
        ActLogNewEntry('  ' + StringGrid1.Cells[3, CurrRow] + LangStatAddFinish, LangAddonsTotal + IntToStr(NumberOfItems));
        ActLogNewEntry('[' + LangMnuAddAddon + ']', LangStatActEnd);
        ActLogSeparator; // Create separator line in logfile

        // Error-Handling
        For n:=1 To 6 Do begin
          if StringGrid1.Cells[n, CurrRow] = 'n/a' then
            StringGrid1.Cells[7, CurrRow] := 'Error';
        end;

        StringGrid1.Row := StringGrid1.RowCount;
        SaveDBToFile(StringGrid1);

        // Make UpdateAddons usable after adding a new AddOn
        cmdUpdateAddons.Enabled := True;
        mUpdateAddons.Enabled := True;

      end;
      AddonUrlList.Free;
      ProgressBar1.Visible := false;

      // Option: Install new Addons immediately (added with 2.1.0)
      if (DirectInstall = true) then
        DoUpdateAddons; // Finally Update Addon
    end;
end;

end;


