{%mainunit umain.pas}

{ <Include-File for uMain - Functions for Database-Upgrades after Website-changes> 
  <WoW Addon-Updater>

  Copyright (C) <2017> <Arndt Braier> <dev@braier.net>

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
}

procedure DBUpgradeCheck; // Check, if DB-Upgrade necessary (e.g. after changes of Addon-Website)
var
  i,
  TotalRows,
  CurrRow: integer;
  SourceText: string = 'Curse'; // Search criteria for needed DB-Update

begin
  with Form1 do begin
    i := 0;
    CurrRow := 1;
    TotalRows := StringGrid1.RowCount - 1;
    for CurrRow := 1 to TotalRows do begin
      if StringGrid1.Cells[6, CurrRow] = SourceText then
        Inc(i); // Count result for this row
    end;
    StringGrid1.Refresh;

    if i > 0 then begin
      // Disable Controls and Menue-Entries
      cmdCheckForAddonUpdates.Enabled := False;
      cmdUpdateAddons.Enabled := False;
      cmdAddNewAddon.Enabled := False;
      cmdRemoveAddon.Enabled := False;
      cmdFilter.Enabled := False;
      mFile.Enabled := False;
      mEdit.Enabled := False;
      mAddons.Enabled := False;
      mUpdateAddons.Enabled := False;
      mSelectAddonDirectory.Enabled := False;
      mShowGnomeAnimations.Enabled := False;
      mSaveOptions.Enabled := False;
      StringGrid1.Enabled := false; // protect StringGrid for changes

      panAttention.Visible := True; // Show Attentionbox
    end;
  end;
end;


procedure DBUpgrade;     // Make DB-Upgrade (e.g. after changes of Addon-Website)
var
  TotalRows,
  CurrRow: integer;
  SourceText: string = 'Curse'; // Search criteria for needed DB-Update
  OldURL: string = 'https://www.curse.com/addons/wow/';
  NewURL: string = 'https://www.curseforge.com/wow/addons/';
  CellText: string = '';

begin
  with Form1 do begin
    // Rewrite DB-Entries according to new Website requirements
    StringGrid1.Enabled := true; // Make StringGrid changeable

    CurrRow := 1;
    TotalRows := StringGrid1.RowCount - 1;

    // Remove old entries (Version, Status, Download-URL) and set Source
    for CurrRow := 1 to TotalRows do begin
      if StringGrid1.Cells[6, CurrRow] = SourceText then begin
         StringGrid1.Cells[7, CurrRow] := '';
         StringGrid1.Cells[6, CurrRow] := 'Curseforge'; // New search criteria, if DB-Upgrade needed in future
         StringGrid1.Cells[5, CurrRow] := '?';
         StringGrid1.Cells[4, CurrRow] := '?';

         CellText := StringGrid1.Cells[2, CurrRow];

         // Prepare URL-Replacement (change 'http:' to 'https:' and 'mods.' to 'www.')
         if Pos('http:', CellText) <> 0 then
           CellText := StringReplace(CellText, 'http:', 'https:',[rfReplaceAll]);

         if Pos('mods.', CellText) <> 0 then
           CellText := StringReplace(CellText, 'mods.', 'www.',[rfReplaceAll]);

         // URL-Replacement
         if Pos(OldURL, CellText) <> 0 then
           CellText := StringReplace(CellText, OldURL, NewURL,[rfReplaceAll]);

         // Remove Slash on End, if exists
         if CellText[length(CellText)] = '/' then
           setlength(CellText, length(CellText)-1);

         StringGrid1.Cells[2, CurrRow] := CellText;
      end;
    end;

    StringGrid1.Refresh;

    // Enable Controls and Menue-Entries again
    if CheckedForAddonUpdates = true then begin
       cmdUpdateAddons.Enabled := True;
       mUpdateAddons.Enabled := True
    end
    else begin
       cmdUpdateAddons.Enabled := False;
       mUpdateAddons.Enabled := False
    end;

    cmdCheckForAddonUpdates.Enabled := True;
    cmdAddNewAddon.Enabled := True;
    cmdRemoveAddon.Enabled := True;
    cmdFilter.Enabled := True;
    mFile.Enabled := True;
    mEdit.Enabled := True;
    mAddons.Enabled := True;
    mSelectAddonDirectory.Enabled := True;
    mShowGnomeAnimations.Enabled := True;
    mSaveOptions.Enabled := True;

    panAttention.Visible := false; // Hide Attentionbox

    // DB-Upgraded, now checking for Updates
    DoCheckForAddonUpdates;
  end;
end;

