{%mainunit umain.pas}

{ <Include-File for uMain - Filter-Functions for Addon-List> 
  <WoW Addon-Updater>

  Copyright (C) <2017> <Arndt Braier> <dev@braier.net>

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
}

function FilterList(Grid: TStringGrid; Str: String): integer;
var
  c, r, RowH: Integer;
//  i: Integer; // for section "Don't hide rows, when nothing found"
  CellText: String;
  ResultColCount: Longword = 0;
  ResultRowCount: integer = 0;

begin
  // Initialize
  Str := AnsiLowerCase(Str);
  RowH := Grid.RowHeights[0];
  Result := 0;

  for r := 1 to Grid.RowCount - 1 do begin
    for c := 1 to Grid.ColCount - 1 do begin
      CellText := AnsiLowerCase(Grid.Cells[c, r]);
      if Pos(Str, CellText) <> 0 then
        Inc(ResultColCount); // Count results for this row
      if ResultColCount = 0 then // Show resulting rows, hide other
        Grid.RowHeights[r] := 0
      else
        Grid.RowHeights[r] := RowH;
    end;
    if ResultColCount > 0 then
      Inc(ResultRowCount);
    ResultColCount := 0; // Reset Column result counter
  end;

{  // Don't hide rows, when nothing found

  // Currently only resulting rows should be displayed - list looks emty,
  // when nothing is found.
  // Next lines commented out, because not needed for now

  if ResultRowCount = 0 then
    for i := 1 to Grid.RowCount - 1 do begin
      Grid.RowHeights[i] := RowH
    end;
}

  if Grid.RowCount > 1 then // Jump to first entry, if availible
    Grid.Row := 1;

  Result := ResultRowCount;

end;


procedure UnfilterList(Grid: TStringGrid);
var
  i, h: integer;
begin
  h := Grid.RowHeights[0];
  for i := 1 to Grid.RowCount - 1 do begin
    Grid.RowHeights[i] := h;
  end;
end;


procedure ResetFilter;   // Reset the Filter Settings
begin
  with Form1 do begin
    FilterSet := false; // Switch Filter off
    StringGrid1.Refresh;
    UnfilterList(StringGrid1);
    lblFilter.Font.Style := [];
    lblFilter.Font.Color := clDefault;
    txtFilterText.Text := '';
    // Don't update Statusbar, if Addon was deleted before
    if Pos(LangStatRemItem, StatusBar1.Panels[0].Text) = 0 then
      StatusBar1.Panels[0].Text := LangStatStart + IntToStr(NumberOfItems);
  end;
end;

procedure TForm1.txtFilterTextChange(Sender: TObject);
var
  AddonsFound: Longword = 0;
begin
  if Length(txtFilterText.Text) > 0 then begin
    FilterSet := true; // Switch Filter on
    // Set graphical options
    if lblFilter.Font.Color <> clBlue then begin
      lblFilter.Font.Style := [fsbold];
      lblFilter.Font.Color := clBlue;
      StringGrid1.Refresh;
    end;
    // Filter Addons and show results
    AddonsFound := FilterList(StringGrid1, txtFilterText.Text);
    StatusBar1.Panels[0].Text := IntToStr(AddonsFound) + LangStatFiltered + IntToStr(NumberOfItems) + '.'
  end
  else
  ResetFilter;
end;


procedure TForm1.txtFilterTextExit(Sender: TObject);
begin
  panFilter.Visible := false;

end;


procedure TForm1.txtFilterTextKeyPress(Sender: TObject; var Key: char);
begin
  if Key = #27 then begin
    if Length(txtFilterText.Text) = 0 then
      panFilter.Visible := false
    else
      ResetFilter
  end;
end;


procedure TForm1.txtFilterTextButtonClick(Sender: TObject);
begin
  ResetFilter;

end;


procedure ShowFilterBox; // Displays the Filter Box
begin
    with Form1 do begin
      // when FilterBox is not there, turn it on
      if panFilter.Visible = false then begin
        panFilter.Visible := true;
        txtFilterText.SetFocus()
      end
      // when FilterBox is there, switch it off
      else begin
        panFilter.Visible := false
      end;
    end;
end;
