{%mainunit umain.pas}

{ <Include-File for uMain - Section Functions for Addon-List> 
  <WoW Addon-Updater>

  Copyright (C) <2016-2017> <Arndt Braier> <dev@braier.net>

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
}

procedure CalcListItems;
begin
    with Form1 do
    begin
        If StringGrid1.RowCount > 0 then
           NumberOfItems := StringGrid1.RowCount - 1;
    end;
end;


// Calculate selected new items, will be used for percentage and progress calculation
function CountNewSelectedAddons(const Grid : TStringGrid): word;
var
  CurrRow: integer = 1;
  Counted: integer = 0;
begin
  for CurrRow := 1 to (Grid.RowCount - 1) do begin
    if (Grid.Cells[1, CurrRow] = '1') and (Grid.Cells[7, CurrRow] = 'New') then
       inc(Counted);
  end;
  Result := Counted;
end;


procedure LoadAddonList;  // Load StringGrid to Array AddonTable
var
  CurrRow: integer = 1;
  CurrCol: integer = 0;

begin
  with Form1 do begin
      try
        SetLength(AddonTable, StringGrid1.RowCount); // Set Array-Size
        // Load StringGrid to array
        for CurrRow := Low(AddonTable) to High(AddonTable) do begin
          for CurrCol := 0 to TotalCols do
            AddonTable[CurrRow, CurrCol] := StringGrid1.Cells[CurrCol, CurrRow];
        end;
      except
        MessageDlg('Error', 'Error in Procedure LoadAddonList.', mtError, [mbOK], 0);
      end;
  end;
end;

procedure SaveAddonList;  // Put Array AddonTable back to StringGrid
var
  CurrRow: integer = 1;
  CurrCol: integer = 0;
  TotalRows: integer = 1;

begin
  with Form1 do begin
      try
        // Delete StringGrid Content
        TotalRows := StringGrid1.RowCount - 1;
        for CurrRow := TotalRows downto 1 do
          GridDeleteRow(StringGrid1, CurrRow);
        // Write array to StringGrid
        CurrRow := StringGrid1.RowCount;
        for CurrRow := CurrRow to High(AddonTable) do begin
          StringGrid1.Row := StringGrid1.RowCount;
          StringGrid1.RowCount := StringGrid1.RowCount + 1;
          for CurrCol := 0 to TotalCols do
            StringGrid1.Cells[CurrCol, CurrRow] := AddonTable[CurrRow, CurrCol];
        end;
        StringGrid1.Row := 0;
      finally
        SetLength(AddonTable, 0); // Free Memory of AddonTable
      end;
  end;
end;

procedure SaveSelections; // Save user-defined selections in Addon list
var
  CurrRow: integer = 1;
begin
  with Form1 do begin
      try
        SetLength(AddonsSelected, StringGrid1.RowCount); // Set Array-Size
        // Load StringGrid to array
        for CurrRow := Low(AddonsSelected) to High(AddonsSelected) do begin
          AddonsSelected[CurrRow] := StringGrid1.Cells[1, CurrRow];
        end;
      except
        MessageDlg('Error', 'Error in Procedure SaveSelections.', mtError, [mbOK], 0);
      end;
  end;
end;

procedure RestoreSelections; // Restore user-defined selections in Addon list
var
  CurrRow: integer = 1;
begin
  with Form1 do begin
      try
        // Write array to StringGrid
        for CurrRow := Low(AddonsSelected) to High(AddonsSelected) do begin
          StringGrid1.Cells[1, CurrRow] := AddonsSelected[CurrRow];
        end;
      finally
        StringGrid1.Refresh;
        SetLength(AddonsSelected, 0); // Free Memory of AddonTable
      end;
  end;
end;

// Deletes a row from StringGrid. Parameter values are StrinGrid and Index of Row
// for deletion starting from 0. For example: GridDeleteRow(StringGrid1, 1);
procedure GridDeleteRow(const Grid : TStringGrid; RowNumber : Integer);
var
  i : Integer;
begin
  for i := RowNumber to Grid.RowCount - 2 do
    Grid.Rows[i].Assign(Grid.Rows[i+ 1]);
  Grid.Rows[Grid.RowCount-1].Clear;
  Grid.RowCount := Grid.RowCount - 1;
end;


// Sort of Addon List
procedure SortAddonList(const ACol: string);
var
    SelCol: byte;

begin
  case ACol of
     'Number':
       SelCol := 0;
     'Name':
       SelCol := 3;
     'State':
       SelCol := 7;
  end;

with Form1 do
begin
  if AllowSortAddonList = true then begin // Only sort, when allowed
    StringGrid1.SortOrder := soAscending; // or soAscending
    StringGrid1.SortColRow(true, SelCol, StringGrid1.FixedRows, StringGrid1.RowCount-1);
  end;
end;
end;


// Renumber the index of complete list
procedure RenumberList;
var
  TotalRows,
  CurrRow: integer;

begin
  with Form1 do
  begin
    CurrRow := 1;
    TotalRows := StringGrid1.RowCount - 1;

    for CurrRow := 1 to TotalRows do
      StringGrid1.Cells[0, CurrRow] := IntToStr(CurrRow);
  end;
end;


procedure SelectAll;
var
  TotalRows,
  CurrRow: integer;

begin
  with Form1 do
  begin
    CurrRow := 1;
    TotalRows := StringGrid1.RowCount - 1;

    for CurrRow := 1 to TotalRows do
      StringGrid1.Cells[1, CurrRow] := '1';
    StringGrid1.Refresh;
  end;
end;


procedure SelectNone;
var
  TotalRows,
  CurrRow: integer;

begin
  with Form1 do
  begin
    CurrRow := 1;
    TotalRows := StringGrid1.RowCount - 1;

    for CurrRow := 1 to TotalRows do
      StringGrid1.Cells[1, CurrRow] := '0';
    StringGrid1.Refresh;
  end;
end;


// Set Sort Options (Sort column 0 and 1 as integer (No. and Selection), rest as strings
procedure TForm1.StringGrid1CompareCells(Sender: TObject; ACol, ARow, BCol,
    BRow: Integer; var Result: integer);
begin
  if (ACol = 0) or (ACol = 1) then begin
     // Result will be either <0, =0, or >0 for normal order.
     result := StrToIntDef(StringGrid1.Cells[ACol,ARow],0)-StrToIntDef(StringGrid1.Cells[BCol,BRow],0);
     // For inverse order, just negate the result (eg. based on grid's SortOrder).
     if StringGrid1.SortOrder = soDescending then
       result := -result;
  end
  else begin
    // Compare strings
        if TStringGrid(Sender).SortOrder = soAscending then
          Result := CompareStr(TStringGrid(Sender).Cells[ACol, ARow], TStringGrid(Sender).Cells[BCol, BRow])
        else
          Result := - CompareStr(TStringGrid(Sender).Cells[ACol, ARow], TStringGrid(Sender).Cells[BCol, BRow]);
  end;
end;


// Popup Menue
procedure TForm1.StringGrid1MouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
var
  iCol: integer = 0;
  iRow: integer = 0;
  CurrRow: integer;

begin
  if Button = mbRight then begin
     with StringGrid1 do
     begin
       MouseToCell(x,y,iCol,iRow);
       Col := iCol;
       Row := iRow;
     end;

     // Initialize
     CurrRow := iRow;
     InfoDiaAddName := '';
     InfoDiaAddVer := '';
     InfoDiaAddURL := '';
     InfoDiaAddChanges := '';

     if CurrRow >= 1 then begin
       InfoDiaAddName := StringGrid1.Cells[3, CurrRow];
       InfoDiaAddVer := StringGrid1.Cells[4, CurrRow];
       InfoDiaAddURL := StringGrid1.Cells[2, CurrRow];

       pmAddonName.Caption := InfoDiaAddName + #32#32#32 + InfoDiaAddVer;

       case MainLanguage of
       'de':
         begin
           pmCheckForUpdate.Caption := InfoDiaAddName + ' ' + LangPMnuChkAddUpd;
           pmUpdate.Caption := InfoDiaAddName + ' ' + LangPMnuUpdAdd;
           pmRem.Caption := InfoDiaAddName + ' ' + LangPMnuRemAddon;
         end;
       'en':
         begin
           pmCheckForUpdate.Caption := LangPMnuChkAddUpd + ' ' + InfoDiaAddName;
           pmUpdate.Caption := LangPMnuUpdAdd + ' ' + InfoDiaAddName;
           pmRem.Caption := LangPMnuRemAddon + ' ' + InfoDiaAddName;
         end;
       end;
       if StringGrid1.Cells[1, CurrRow] = '1' then begin
         pmCheckForUpdate.Visible := true;
         pmUpdate.Visible := true;
         pmSplit3.Visible := true;
       end
       else begin
         pmCheckForUpdate.Visible := false;
         pmUpdate.Visible := false;
         pmSplit3.Visible := false;
       end;
       popmnuMain.Popup(Mouse.CursorPos.x,Mouse.CursorPos.y);
     end;
  end;
end;


procedure TForm1.StringGrid1PrepareCanvas(sender: TObject; aCol, aRow: Integer;
    aState: TGridDrawState);
var
  ts: TTextStyle;

begin
  // Visualisation options for Grid (Colors and Font-Weight)

  // Separate every second line in gray color
  if (ACol > 0) And (ARow > 0) And (ARow mod 2 = 0) then
    StringGrid1.Canvas.Brush.Color := clBtnFace;

  // Colorize Cells according State-Column
  // New = Yellow
  // Updated = Green
  // OK / None = White
  // Error = Red
  // Selected = Font Bold

  // Align Text to Center in Columns Number and Status
  if (ACol = 0) or (ACol = 7) then begin
    ts := StringGrid1.Canvas.TextStyle;
    ts.Alignment := taCenter;
    StringGrid1.Canvas.TextStyle := ts;
  end;

  // Colour cells, if Addon is selected
  if (StringGrid1.Cells[7, ARow] = 'New') AND (StringGrid1.Cells[1, ARow] = '1') then
    StringGrid1.Canvas.Brush.Color := clYellow;

  if (StringGrid1.Cells[7, ARow] = 'Updated') AND (StringGrid1.Cells[1, ARow] = '1') then
    StringGrid1.Canvas.Brush.Color := clLime;

  if (StringGrid1.Cells[7, ARow] = 'Error') AND (StringGrid1.Cells[1, ARow] = '1') then // Added with 2.1.0
    StringGrid1.Canvas.Brush.Color := clRed;

  if StringGrid1.Cells[1, ARow] = '1' then
    StringGrid1.Canvas.Font.Style := [fsBold];

  // Non selected Addon rows in grey colour, other text in grid black
  if StringGrid1.Cells[1, ARow] = '0' then begin
    StringGrid1.Canvas.Font.Color := clGray;
    StringGrid1.Canvas.Font.Style := [fsItalic];
  end
  else
    StringGrid1.Canvas.Font.Color := clBlack;

  // When Filter is active, set top row to blue and bold
  if (ARow = 0) AND (FilterSet = true) then begin
    StringGrid1.Canvas.Font.Color := clBlue;
    StringGrid1.Canvas.Font.Style := [fsBold]
  end;

  if (ARow = 0) AND (FilterSet = false) then begin
    StringGrid1.Canvas.Font.Color := clBlack;
    StringGrid1.Canvas.Font.Style := []
  end;

  // Selected row with blue background and white text
  if (gdSelected in aState) or (gdFocused in aState) then begin
    StringGrid1.Canvas.Brush.Color := clBlue;
    StringGrid1.Canvas.Font.Color := clWhite;
  end;

end;


procedure TForm1.StringGrid1SelectCell(Sender: TObject; aCol, aRow: Integer;
  var CanSelect: Boolean);
begin
  //
  if aCol = 1 then // Write-protection for column 1 (Select column)
    StringGrid1.Options:= StringGrid1.Options +[goEditing]
  else
    StringGrid1.Options:= StringGrid1.Options -[goEditing]
end;


