unit ProgramConfig;

{ <WoW Addon-Updater>
  <Functions to create and store User Application settings>

  Copyright (C) <2016-2017> <Arndt Braier> <dev@braier.net>

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
}

{$mode objfpc}{$H+}

interface

uses
    Classes, SysUtils, INIFiles, FileUtil;

procedure saveAppConfig(
  WindowPosLeft,
  WindowPosTop,
  WindowHeight,
  WindowWidth,
  ColWidthA,
  ColWidthB,
  ColWidthC,
  ColWidthD,
  ColWidthE,
  ColWidthF,
  ColWidthG,
  SetGnomeAnimations,
  SetDirectInstall: integer;
  Language,
  AppDataPath: string);

procedure loadAppConfig(
  var WindowPosLeft,
  WindowPosTop,
  WindowHeight,
  WindowWidth,
  ColWidthA,
  ColWidthB,
  ColWidthC,
  ColWidthD,
  ColWidthE,
  ColWidthF,
  ColWidthG,
  SetGnomeAnimations,
  SetDirectInstall: integer;
  var Language,
  AppDataPath: string);


implementation


procedure saveAppConfig(
  WindowPosLeft,
  WindowPosTop,
  WindowHeight,
  WindowWidth,
  ColWidthA,
  ColWidthB,
  ColWidthC,
  ColWidthD,
  ColWidthE,
  ColWidthF,
  ColWidthG,
  SetGnomeAnimations,
  SetDirectInstall: integer;
  Language,
  AppDataPath: string);
var
  Ini: TIniFile;
  IniFile: string;

begin
  IniFile := GetAppConfigDir(false) + 'config.ini';
  Ini:=TIniFile.Create(IniFile);

  try
    Ini.WriteInteger('Window', 'WindowPosLeft', WindowPosLeft);
    Ini.WriteInteger('Window', 'WindowPosTop', WindowPosTop);
    Ini.WriteInteger('Window', 'WindowHeight', WindowHeight);
    Ini.WriteInteger('Window', 'WindowWidth', WindowWidth);
    Ini.WriteInteger('Table', 'Column1', ColWidthA);
    Ini.WriteInteger('Table', 'Column2', ColWidthB);
    Ini.WriteInteger('Table', 'Column3', ColWidthC);
    Ini.WriteInteger('Table', 'Column4', ColWidthD);
    Ini.WriteInteger('Table', 'Column5', ColWidthE);
    Ini.WriteInteger('Table', 'Column6', ColWidthF);
    Ini.WriteInteger('Table', 'Column7', ColWidthG);
    Ini.WriteInteger('Options', 'GnomeAnimations', SetGnomeAnimations);
    Ini.WriteInteger('Options', 'DirectInstall', SetDirectInstall);
    Ini.WriteString('Localization', 'Language', Language);
    Ini.WriteString('Path', 'AppDataPath', AppDataPath);
    //Ini.WriteBool('Section', 'Schluessel', True);

  finally
    Ini.Free;

  end;

end;

procedure loadAppConfig(
  var WindowPosLeft,
  WindowPosTop,
  WindowHeight,
  WindowWidth,
  ColWidthA,
  ColWidthB,
  ColWidthC,
  ColWidthD,
  ColWidthE,
  ColWidthF,
  ColWidthG,
  SetGnomeAnimations,
  SetDirectInstall: integer;
  var Language,
  AppDataPath: string);
var
  Ini: TIniFile;
  IniFile: string;

begin

  IniFile := GetAppConfigDir(false) + 'config.ini';
  Ini:=TIniFile.Create(IniFile);

  try
    WindowPosLeft := Ini.ReadInteger('Window', 'WindowPosLeft', 20);
    WindowPosTop := Ini.ReadInteger('Window', 'WindowPosTop', 20);
    WindowHeight := Ini.ReadInteger('Window', 'WindowHeight', 600);
    WindowWidth := Ini.ReadInteger('Window', 'WindowWidth', 800);
    ColWidthA := Ini.ReadInteger('Table', 'Column1', 60);
    ColWidthB := Ini.ReadInteger('Table', 'Column2', 175);
    ColWidthC := Ini.ReadInteger('Table', 'Column3', 125);
    ColWidthD := Ini.ReadInteger('Table', 'Column4', 75);
    ColWidthE := Ini.ReadInteger('Table', 'Column5', 150);
    ColWidthF := Ini.ReadInteger('Table', 'Column6', 150);
    ColWidthG := Ini.ReadInteger('Table', 'Column7', 75);
    SetGnomeAnimations := Ini.ReadInteger('Options', 'GnomeAnimations', 1);
    SetDirectInstall := Ini.ReadInteger('Options', 'DirectInstall', 1);
    Language := Ini.ReadString('Localization', 'Language', 'en');
    AppDataPath := Ini.ReadString('Path', 'AppDataPath', 'not defined');

  finally
    Ini.Free;

  end;

end;


end.

