unit ProgramData;

{ <WoW Addon-Updater>
  <Functions to handle Application created data>

  Copyright (C) <2016-2017> <Arndt Braier> <dev@braier.net>

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
}

{$mode objfpc}{$H+}

interface

uses
    Classes, SysUtils, Grids, INIFiles, FileUtil;

procedure LoadDBFromFile(StringGrid: TStringGrid);
procedure SaveDBToFile(StringGrid: TStringGrid);

implementation


procedure LoadDBFromFile(StringGrid: TStringGrid);
var
  FileName: string;
begin
  StringGrid.SaveOptions := [soDesign, soContent];
  {
  soDesign:     Save & load ColCount,RowCount,FixedCols,FixedRows,
                ColWidths, RowHeights and Options (TCustomGrid)
  soPosition:   Save & load Scroll position, Row, Col and Selection (TCustomGrid)
  soAttributes: Save & load Colors, Text Alignment & Layout, etc. (TCustomDrawGrid)
  soContent:    Save & load Text (TCustomStringGrid)
  soAll:        set of all options (soAll:=[soDesign,soPosition,soAttributes,soContent];)
  }
  FileName := GetAppConfigDir(false) + 'addondb.xml';

  if FileExists(FileName) then begin // check, if file exist, otherwise create it.
      StringGrid.Clear;
      try
        StringGrid.LoadFromFile(FileName);
      finally
        StringGrid.Refresh;
      end;
    end
  else
    begin
      try
        StringGrid.SaveToFile(FileName);
      finally
        StringGrid.Refresh;
      end;
    end;
end;


procedure SaveDBToFile(StringGrid: TStringGrid);
var
  FileName,
  BackupFileName: string;
  BackupIni: TIniFile;
  BackupIniFile: string;
  BackupCount: Byte = 0;

const
  MaxBackups: Byte = 24;

begin
  StringGrid.SaveOptions := [soDesign, soContent];
  {
  soDesign:     Save & load ColCount,RowCount,FixedCols,FixedRows,
                ColWidths, RowHeights and Options (TCustomGrid)
  soPosition:   Save & load Scroll position, Row, Col and Selection (TCustomGrid)
  soAttributes: Save & load Colors, Text Alignment & Layout, etc. (TCustomDrawGrid)
  soContent:    Save & load Text (TCustomStringGrid)
  soAll:        set of all options (soAll:=[soDesign,soPosition,soAttributes,soContent];)
  }
  FileName := GetAppConfigDir(false) + 'addondb.xml';

  // Implementation of backup-routine
  // Before save, makes an copy to a .1, .2, .n file

  if FileExists(FileName) then begin
      // Only start, when addondb.xml exists
      // Looking for existing backup-files

      // Initialize Variables
      BackupIniFile := GetAppConfigDir(false) + 'backupcount.ini';
      BackupIni:=TIniFile.Create(BackupIniFile);

      // Get Number of current backup from ini-file
      try
        BackupCount := BackupIni.ReadInteger('BackupCounter', 'CurrentBackup', 0);
      finally
        BackupIni.Free;
      end;

      // If Backup Counter reached MaxBackups, set it back to 0
      if BackupCount = MaxBackups then
         BackupCount := 0;

      // Increment Backup Counter
      BackupCount := BackupCount + 1;

      // Create Backup-Copy of DB-File
      BackupFileName := FileName + '.' + IntToStr(BackupCount);
      CopyFile(FileName,BackupFileName);

      // Save Number of current Backup to ini-file
      BackupIni:=TIniFile.Create(BackupIniFile);
      try
        BackupIni.WriteInteger('BackupCounter', 'CurrentBackup', BackupCount);
      finally
        BackupIni.Free;
      end;
  end;

  // End of backup-routine, let's save the DB ...
  try
    StringGrid.SaveToFile(FileName);
  finally
    StringGrid.Refresh;
  end;
end;


end.

