unit uActLog;

{ <WoW Addon-Updater>
  <Functions for Activity-Log and Protocol>

  Copyright (C) <2016-2017> <Arndt Braier> <dev@braier.net>

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
}

{$mode objfpc}{$H+}

interface

uses
    Classes, SysUtils, Dialogs, FileUtil, FileOperations, uUtils;

procedure ActLogNewEntry(LAction, LContent: string);
procedure ActLogSeparator;
procedure ActLogLoad;
procedure ActLogSave;
procedure ActLogProgEnd;

var
   ActLog: TStringList;            // Contains Activity-Log
   ActLogEnable: boolean = true;   // Switch for Activity-Log

implementation

procedure ActLogNewEntry(LAction, LContent: string);
var
  LogEntry: string;
  LogSpacer: string = ' ';

begin
  if ActLogEnable = true then begin
    LogEntry := '[' + DateTimeToStr(Now) + ']' + LogSpacer + LAction + LogSpacer + LContent;
    ActLog.Add(LogEntry);
    ActLogSave;
  end;
end;

procedure ActLogSeparator;
const
  C: char = '-';

begin
  ActLog.Add(RepeatChar(C,60));  // Create separator line
  ActLogSave;
end;

procedure ActLogLoad;
var
  LogfileSize: integer;
  FileName, LogDir, LogArchiveDir, Delimiter, ArchiveFileName: string;
  FilesToArchiveList: TStringList;

const
  LogfileMaxSize: integer = 2097152; // Limits Logfile to 2 MB

begin
  if ActLogEnable = true then begin
    ActLog := TStringList.Create;
    ActLog.Clear;

    // Define delimiter for Windows (\) or other Systems (/)
    {$IFDEF WINDOWS} // Compiler-Switch
      Delimiter := '\';
    {$ELSE}
      Delimiter := '/';
    {$ENDIF}

    LogDir := GetAppConfigDir(false) + 'logfiles' + Delimiter;
    LogArchiveDir := LogDir + 'archive' + Delimiter;

    FileName := LogDir + 'wau.log';
    ArchiveFileName := LogArchiveDir + 'wau.log.zip';
    FilesToArchiveList := TStringList.Create;
    FilesToArchiveList.Add(FileName);

    // Create Directories in User-Config-Directory, if not exist
    ForceDirectories(LogDir);
    ForceDirectories(LogArchiveDir);

    if FileExists(FileName) then begin // check, if file exist, otherwise create it.
      try begin
        // Checking Size of Logfile and archive it, when its bigger than LogfileMaxSize
        LogfileSize := FileSize(FileName);
        If LogfileSize > LogfileMaxSize then begin
          // Starting Archive Process
          ArchiveFileName := CreateUniqueFileName(ArchiveFileName);
          subZip(ArchiveFileName, FilesToArchiveList);
          ActLogNewEntry('Archived', IntToStr(LogfileSize) + ' Bytes in ' + ArchiveFileName);
          try
            ActLog.SaveToFile(FileName);
          except
            MessageDlg('Saving of Logfile ' + FileName + ' failed! Please check the File/Filesystem for errors.', mtError, [mbOK], 0);
          end;
        end;
        ActLog.LoadFromFile(FileName);
      end;
      except
        MessageDlg('Loading of Logfile ' + FileName + ' failed! Please check the File for errors.', mtError, [mbOK], 0);
      end;
    end
    else begin
      try
        ActLog.SaveToFile(FileName);
      except
        MessageDlg('Saving of Logfile ' + FileName + ' failed! Please check the File/Filesystem for errors.', mtError, [mbOK], 0);
      end;
    end;

    FilesToArchiveList.Free; // Cleanup

    ActLog.Add(RepeatChar('=',60));  // Create separator line
  end;
end;


procedure ActLogSave;
var
  FileName, LogDir, Delimiter: string;

begin
  if ActLogEnable = true then begin
    // Define delimiter for Windows (\) or other Systems (/)
    {$IFDEF WINDOWS} // Compiler-Switch
      Delimiter := '\';
    {$ELSE}
      Delimiter := '/';
    {$ENDIF}

    LogDir := GetAppConfigDir(false) + 'logfiles' + Delimiter;
    FileName := LogDir + 'wau.log';

    try
      ForceDirectories(LogDir); // Create Directory in User-Config-Directory, if not exist
      ActLog.SaveToFile(FileName);
    except
      MessageDlg('Saving of Logfile ' + FileName + ' failed! Please check the File/Filesystem for errors.', mtError, [mbOK], 0);
    end;
  end;
end;


procedure ActLogProgEnd;
begin
  if ActLogEnable = true then begin
    ActLogSave;
    ActLog.Free; // Cleanup
  end;
end;


end.

