unit uLanguage;

{ <WoW Addon-Updater>
  <Language Settings File>

  Copyright (C) <2016-2019> <Arndt Braier> <dev@braier.net>

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
}

{$mode objfpc}{$H+}

interface

uses
  Classes, FileUtil, SysUtils, LCLTranslator;

var
  LangVersion,
  LangVersionInfo,
  LangShortInfo,
  LangErrorLoadDB: string;

resourcestring
  LangAddonsTotal          = 'LangAddonsTotal';
  LangColAddonDownloadUrl  = 'LangColAddonDownloadUrl';
  LangColAddonName         = 'LangColAddonName';
  LangColAddonSource       = 'LangColAddonSource';
  LangColAddonStatus       = 'LangColAddonStatus';
  LangColAddonUrl          = 'LangColAddonUrl';
  LangColAddonVersion      = 'LangColAddonVersion';
  LangColSelect            = 'LangColSelect';
  LangDescAddAddon         = 'LangDescAddAddon';
  LangDescChkAddUpd        = 'LangDescChkAddUpd';
  LangDescHelp             = 'LangDescHelp';
  LangDescRemAddon         = 'LangDescRemAddon';
  LangDescUpdAdd           = 'LangDescUpdAdd';
  LangErrorLoadDBBegin     = 'LangErrorLoadDBBegin';
  LangErrorLoadDBEnd       = 'LangErrorLoadDBEnd';
  LangFirstStart           = 'LangFirstStart';
  LangGrpExpl              = 'LangGrpExpl';
  LangInfoTabTitleA        = 'LangInfoTabTitleA';
  LangInfoTabTitleB        = 'LangInfoTabTitleB';
  LangInfoTabTitleC        = 'LangInfoTabTitleC';
  LangInfoTabTitleD        = 'LangInfoTabTitleD';
  LangLblNewAvail          = 'LangLblNewAvail';
  LangLblUnchanged         = 'LangLblUnchanged';
  LangLblUpdated           = 'LangLblUpdated';
  LangLblError             = 'LangLblError';
  LangMnuAdd               = 'LangMnuAdd';
  LangMnuAddAddon          = 'LangMnuAddAddon';
  LangMnuChkAddUpd         = 'LangMnuChkAddUpd';
  LangMnuDE                = 'LangMnuDE';
  LangMnuEN                = 'LangMnuEN';
  LangMnuEdit              = 'LangMnuEdit';
  LangMnuFile              = 'LangMnuFile';
  LangMnuHelp              = 'LangMnuHelp';
  LangMnuInfo              = 'LangMnuInfo';
  LangMnuLanguage          = 'LangMnuLanguage';
  LangMnuOpen              = 'LangMnuOpen';
  LangMnuOptions           = 'LangMnuOptions';
  LangMnuQuit              = 'LangMnuQuit';
  LangMnuRemAddon          = 'LangMnuRemAddon';
  LangMnuSaveDB            = 'LangMnuSaveDB';
  LangMnuSaveSettings      = 'LangMnuSaveSettings';
  LangMnuSelAll            = 'LangMnuSelAll';
  LangMnuSelNone           = 'LangMnuSelNone';
  LangMnuSetAddonDir       = 'LangMnuSetAddonDir';
  LangMnuShortHelp         = 'LangMnuShortHelp';
  LangMnuSortName          = 'LangMnuSortName';
  LangMnuUpdAdd            = 'LangMnuUpdAdd';
  LangMsgBoxDelAddonText   = 'LangMsgBoxDelAddonText';
  LangMsgBoxDelSingleAddonText = 'LangMsgBoxDelSingleAddonText';
  LangMsgBoxDelAddonTitle  = 'LangMsgBoxDelAddonTitle';
  LangReqBoxAddonUrlText   = 'LangReqBoxAddonUrlText';
  LangReqBoxAddonUrlTitle  = 'LangReqBoxAddonUrlTitle';
  LangSelAddonDirDialog    = 'LangSelAddonDirDialog';
  LangShortHelp            = 'LangShortHelp';
  LangShortInfoBegin       = 'LangShortInfoBegin';
  LangShortInfoEnd         = 'LangShortInfoEnd';
  LangStatAddFinish        = 'LangStatAddFinish';
  LangStatAddStepA         = 'LangStatAddStepA';
  LangStatAddStepB         = 'LangStatAddStepB';
  LangStatAddStepC         = 'LangStatAddStepC';
  LangStatAddStepD         = 'LangStatAddStepD';
  LangStatCheckUpdate      = 'LangStatCheckUpdate';
  LangStatCheckUpdateDone  = 'LangStatCheckUpdateDone';
  LangStatFinished         = 'LangStatFinished';
  LangStatLoadedDB         = 'LangStatLoadedDB';
  LangStatRemItem          = 'LangStatRemItem';
  LangStatSavedDB          = 'LangStatSavedDB';
  LangStatSavedSettings    = 'LangStatSavedSettings';
  LangStatSetDirectory     = 'LangStatSetDirectory';
  LangStatStart            = 'LangStatStart';
  LangStatUpdateAddon      = 'LangStatUpdateAddon';
  LangStatUpdateAddonDone  = 'LangStatUpdateAddonDone';
  LangStatusbarAddonText   = 'LangStatusbarAddonText';
  LangVersionInfoText      = 'LangVersionInfoText';
  LangMnuShowGnomeAnimations = 'LangMnuShowGnomeAnimations';
  LangMnuDirectInstall     = 'LangMnuDirectInstall';
  LangStatLogStart         = 'LangStatLogStart';
  LangStatLogEnd           = 'LangStatLogEnd';
  LangStatActStart         = 'LangStatActStart';
  LangStatActEnd           = 'LangStatActEnd';
  LangStatLogNewAddVer     = 'LangStatLogNewAddVer';
  LangStatRem              = 'LangStatRem';
  LangStatWas              = 'LangStatWas';
  LangPMnuOpenWebsite      = 'LangPMnuOpenWebsite';
  LangPMnuWebSearch        = 'LangPMnuWebSearch';
  LangPMnuChkAddUpd        = 'LangPMnuChkAddUpd';
  LangPMnuUpdAdd           = 'LangPMnuUpdAdd';
  LangPMnuRemAddon         = 'LangPMnuRemAddon';
  LangPMnuAddonChanges     = 'LangPMnuAddonChanges';
  LangFilterTextHint       = 'LangFilterTextHint';
  LangFilterTextButtonHint = 'LangFilterTextButtonHint';
  LangGrpFilter            = 'LangGrpFilter';
  LangLblFilter            = 'LangLblFilter';
  LangMnuFilter            = 'LangMnuFilter';
  LangStatFiltered         = 'LangStatFiltered';
  LangLblShortVersionInfo  = 'LangLblShortVersionInfo';
  LangNothingFound         = 'LangNothingFound';
  LangRetrieve             = 'LangRetrieve';
  LanglblAttentionMain     = 'LanglblAttentionMain';
  LanglblAttentionDesc     = 'LanglblAttentionDesc';
  LangcmdConfirm           = 'LangcmdConfirm';
  LangAddNewlblResults     = 'LangAddNewlblResults';
  LangAddNewlblAddNew      = 'LangAddNewlblAddNew';
  LangAddNewGrpSearch      = 'LangAddNewGrpSearch';
  LangAddNewChkDirectUrl   = 'LangAddNewChkDirectUrl';
  LangAddNewChkSearchEng   = 'LangAddNewChkSearchEng';
  LangAddNewChkSearchEngHint = 'LangAddNewChkSearchEngHint';
  LangAddNewCmdSearch      = 'LangAddNewCmdSearch';
  LangAddNewRadioGroup1    = 'LangAddNewRadioGroup1';
  LangAddNewGrpResult      = 'LangAddNewGrpResult';
  LangAddNewGrpResultHint  = 'LangAddNewGrpResultHint';
  LangAddNewGrpAddNew      = 'LangAddNewGrpAddNew';
  LangAddNewListBox1Hint   = 'LangAddNewListBox1Hint';


procedure SetLanguageSettings(lang: string);  // actually accepts 'en' and 'de'

implementation

procedure SetLanguageSettings(lang: string);
begin
  SetDefaultLang(lang);
  LangVersion := 'WoW-Addon-Updater 3.1.0 (Vivienna)' + #13#10 + #13#10 +
    'Copyright (C) <2016-2019> <Arndt Braier> <dev@braier.net>' + #13#10 + #13#10;
  LangVersionInfo := LangVersion + LangVersionInfoText;
  LangShortInfo   := LangVersion + LangShortInfoBegin + GetAppConfigDir(false) + #13#10 + #13#10 + LangShortInfoEnd;
  LangErrorLoadDB := LangErrorLoadDBBegin + GetAppConfigDir(false) + LangErrorLoadDBEnd;
end;

end.

