unit uMain;

{ <WoW Addon-Updater>

  Copyright (C) <2016-2019> <Arndt Braier> <dev@braier.net>

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
}

{$mode objfpc}{$H+}

interface

uses
    Classes, SysUtils, FileUtil, Forms, Controls, Graphics, Dialogs, StdCtrls,
    ExtCtrls, ComCtrls, Menus, Grids, Buttons, LCLType, EditBtn, AddonUtils,
    ProgramConfig, ProgramData, uLanguage, uInfo, uHelp, uAniWave, uStatus,
    uUtils, uAddonChanges, math, DateUtils, uActLog, uAddNew, LCLIntf;

type

    { TChkAddUpdThread }
    TStatusChkAddUpdEvent = procedure(CurrPosValue: integer; AddonStatusValue, AddonNameValue, AddonProgressValue: String) of Object;

    TChkAddUpdThread = class(TThread)
    private
      fCurrPos: integer;
      fAddonStatusText,
      fAddonNameText,
      fAddonProgressText: string;
      FOnStatusChkAddUpd: TStatusChkAddUpdEvent;
      procedure StatusChkAddUpd;
    protected
      procedure Execute; override;
    public
      Constructor Create(CreateSuspended : boolean);
      property OnStatusChkAddUpd: TStatusChkAddUpdEvent read FOnStatusChkAddUpd write FOnStatusChkAddUpd;
    end;


    { TDoAddUpdThread }
    TStatusDoAddUpdEvent = procedure(CurrPosValue: integer; AddonStatusValue, AddonNameValue, AddonProgressValue: String) of Object;

    TDoAddUpdThread = class(TThread)
    private
      fCurrPos: integer;
      fAddonStatusText,
      fAddonNameText,
      fAddonProgressText: string;
      FOnStatusDoAddUpd: TStatusDoAddUpdEvent;
      procedure StatusDoAddUpd;
    protected
      procedure Execute; override;
    public
      Constructor Create(CreateSuspended : boolean);
      property OnStatusDoAddUpd: TStatusDoAddUpdEvent read FOnStatusDoAddUpd write FOnStatusDoAddUpd;
    end;


    { TForm1 }

    TForm1 = class(TForm)
      cmdConfirm: TButton;
        cmdFilter: TBitBtn;
        cmdInfo: TBitBtn;
        cmdHelp: TBitBtn;
        Button1: TButton;
        cmdAddNewAddon: TBitBtn;
        cmdRemoveAddon: TBitBtn;
        cmdCheckForAddonUpdates: TBitBtn;
        cmdUpdateAddons: TBitBtn;
        imgGnomeTell: TImage;
        lblAttentionDesc: TLabel;
        lblError: TLabel;
        lblAttentionMain: TLabel;
        mDirectInstall: TMenuItem;
        panAttention: TPanel;
        pmAddonChanges: TMenuItem;
        pmWebSearch: TMenuItem;
        pmOpenWebsite: TMenuItem;
        pmAddonName: TMenuItem;
        pmSplit1: TMenuItem;
        pmSplit3: TMenuItem;
        panFilter1: TPanel;
        shaError: TShape;
        shaAttention: TShape;
        txtFilterText: TEditButton;
        panFilter: TPanel;
        grpFilter: TGroupBox;
        grpExplanation: TGroupBox;
        imgGnomeWave: TImage;
        imgGnomeRun: TImage;
        ImageListGnomeWave: TImageList;
        ImageListGnomeRun: TImageList;
        imgGradient: TImage;
        ImageList1: TImageList;
        lblInfo: TLabel;
        lblProgress: TLabel;
        lblAddonName: TLabel;
        lblFilter: TLabel;
        lblStatus: TLabel;
        lblAddNewAddon: TLabel;
        lblRemoveAddon: TLabel;
        lblCheckForAddonUpdates: TLabel;
        lblUpdateAddons: TLabel;
        lblHelp: TLabel;
        lblNewVersionAvail: TLabel;
        lblAddonUpdated: TLabel;
        lblUnchanged: TLabel;
        mEdit: TMenuItem;
        MenuItem1: TMenuItem;
        mDE: TMenuItem;
        mEN: TMenuItem;
        mAddons: TMenuItem;
        mAddNewAddon: TMenuItem;
        MenuItem2: TMenuItem;
        mCheckForAddonUpdates: TMenuItem;
        MenuItem3: TMenuItem;
        MenuItem4: TMenuItem;
        mFilter: TMenuItem;
        pmRem: TMenuItem;
        pmCheckForUpdate: TMenuItem;
        pmUpdate: TMenuItem;
        pmSplit2: TMenuItem;
        mShowGnomeAnimations: TMenuItem;
        mSortName: TMenuItem;
        mSelAll: TMenuItem;
        mSelNone: TMenuItem;
        mUpdateAddons: TMenuItem;
        mRemoveAddon: TMenuItem;
        mSaveDB: TMenuItem;
        mLanguage: TMenuItem;
        mOptions: TMenuItem;
        mInfo: TMenuItem;
        mShortHelp: TMenuItem;
        mHelp: TMenuItem;
        mSelectAddonDirectory: TMenuItem;
        mSaveOptions: TMenuItem;
        mOpenDB: TMenuItem;
        mExit: TMenuItem;
        mnuMain: TMainMenu;
        mFile: TMenuItem;
        panGnomeWave: TPanel;
        panStatus: TPanel;
        popmnuMain: TPopupMenu;
        ProgressBar1: TProgressBar;
        SelectAddonDirectoryDialog: TSelectDirectoryDialog;
        shaNewVersionAvail: TShape;
        shaAddonUpdated: TShape;
        Shape1: TShape;
        shaStatus: TShape;
        shaUnchanged: TShape;
        StatusBar1: TStatusBar;
        StatusBar2: TStatusBar;
        StringGrid1: TStringGrid;
        TimerStatusClose: TTimer;
        TimerAniWave: TTimer;
        TimerAniRun: TTimer;
        procedure Button1Click(Sender: TObject);
        procedure cmdAddNewAddonClick(Sender: TObject);
        procedure cmdCheckForAddonUpdatesClick(Sender: TObject);
        procedure cmdConfirmClick(Sender: TObject);
        procedure cmdFilterClick(Sender: TObject);
        procedure cmdHelpClick(Sender: TObject);
        procedure cmdHelpMouseEnter(Sender: TObject);
        procedure cmdHelpMouseLeave(Sender: TObject);
        procedure cmdInfoClick(Sender: TObject);
        procedure cmdRemoveAddonClick(Sender: TObject);
        procedure cmdUpdateAddonsClick(Sender: TObject);
        procedure FormClose(Sender: TObject; var CloseAction: TCloseAction);
        procedure FormCloseQuery(Sender: TObject; var CanClose: boolean);
        procedure FormCreate(Sender: TObject);
        procedure FormResize(Sender: TObject);
        procedure imgGradientClick(Sender: TObject);
        procedure mAddNewAddonClick(Sender: TObject);
        procedure mCheckForAddonUpdatesClick(Sender: TObject);
        procedure mDEClick(Sender: TObject);
        procedure mDirectInstallClick(Sender: TObject);
        procedure mENClick(Sender: TObject);
        procedure mExitClick(Sender: TObject);
        procedure mFilterClick(Sender: TObject);
        procedure mInfoClick(Sender: TObject);
        procedure mOpenDBClick(Sender: TObject);
        procedure mRemoveAddonClick(Sender: TObject);
        procedure mSaveDBClick(Sender: TObject);
        procedure mSaveOptionsClick(Sender: TObject);
        procedure mSelAllClick(Sender: TObject);
        procedure mSelectAddonDirectoryClick(Sender: TObject);
        procedure mSelNoneClick(Sender: TObject);
        procedure mShortHelpClick(Sender: TObject);
        procedure mShowGnomeAnimationsClick(Sender: TObject);
        procedure mSortNameClick(Sender: TObject);
        procedure mUpdateAddonsClick(Sender: TObject);
        procedure pmAddonChangesClick(Sender: TObject);
        procedure pmCheckForUpdateClick(Sender: TObject);
        procedure pmOpenWebsiteClick(Sender: TObject);
        procedure pmRemClick(Sender: TObject);
        procedure pmUpdateClick(Sender: TObject);
        procedure pmWebSearchClick(Sender: TObject);
        procedure StringGrid1CompareCells(Sender: TObject; ACol, ARow, BCol,
            BRow: Integer; var Result: integer);
        procedure StringGrid1MouseDown(Sender: TObject; Button: TMouseButton;
          Shift: TShiftState; X, Y: Integer);
        procedure StringGrid1PrepareCanvas(sender: TObject; aCol,
            aRow: Integer; aState: TGridDrawState);
        procedure StringGrid1SelectCell(Sender: TObject; aCol, aRow: Integer;
          var CanSelect: Boolean);
        procedure TimerAniRunTimer(Sender: TObject);
        procedure TimerAniWaveTimer(Sender: TObject);
        procedure TimerStatusCloseTimer(Sender: TObject);
        procedure txtFilterTextButtonClick(Sender: TObject);
        procedure txtFilterTextChange(Sender: TObject);
        procedure txtFilterTextExit(Sender: TObject);
        procedure txtFilterTextKeyPress(Sender: TObject; var Key: char);
    private
        { private declarations }
      ChkAddUpdThread: TChkAddUpdThread;
      DoAddUpdThread: TDoAddUpdThread;
      procedure StatusChkAddUpd(CurrPosValue: integer; AddonStatusValue, AddonNameValue, AddonProgressValue: String);
      procedure OnDoneChkAddUpd(ASender : TObject);
      procedure StatusDoAddUpd(CurrPosValue: integer; AddonStatusValue, AddonNameValue, AddonProgressValue: String);
      procedure OnDoneDoAddUpd(ASender : TObject);
    public
        { public declarations }
      AddonUrlList: TStringList; // Handover from Add New Addon - Dialog-Window
      AddNewResult: boolean; // Result of Add New Addon - Dialog-Window
    end;

procedure ApplyLanguage;
procedure CalcListItems;
procedure FillOutAddonTable;
procedure GridDeleteRow(const Grid : TStringGrid; RowNumber : Integer);
procedure RenumberList;
procedure SaveUserConfig;
procedure SelectAll;
procedure SelectNone;
procedure SortAddonList(const ACol: string);
procedure DrawUpperGradient;
procedure StatusBoxEnable;
procedure StatusBoxDisable;
procedure StatusBoxUpdate;
procedure LoadAddonList;  // Load StringGrid to Array AddonTable
procedure SaveAddonList;  // Put Array AddonTable back to StringGrid
procedure SaveSelections; // Save user-defined selections in Addon list
procedure RestoreSelections; // Restore user-defined selections in Addon list
procedure ShowFilterBox; // Displays the Filter Box
procedure ResetFilter;   // Reset the Filter Settings
procedure UnfilterList(Grid: TStringGrid); // Restore Addon list after filtering
procedure OpenWebsite;   // Open the Addon related website in default browser
procedure WebSearch;     // Search Addon with Search engine in default browser
procedure DBUpgradeCheck; // Check, if DB-Upgrade necessary (e.g. after changes of Addon-Website)
procedure DBUpgrade;     // Make DB-Upgrade (e.g. after changes of Addon-Website)

function IsStringInteger(const AString: String): Boolean;
function CountNewSelectedAddons(const Grid : TStringGrid): word;
function FilterList(Grid: TStringGrid; Str: String): integer; // Filter Addon list


const
    TotalCols: integer = 7;             // Number of columns in Addon Table

var
    Form1: TForm1;
    AddonDir: string = 'not defined';   // Defines the World of Warcraft/Interface/AddOns Directory
    MainLanguage: string = 'en';        // can be set to de, en, etc.
    NumberOfItems: word = 0;            // Number of installed Addons
    NumberOfNewSelItems: word = 0;      // Number of selected Addons
    ChangesCounter: word = 0;           // Resulting Addon entry changes after function processing
    AddonTable: array of array[0..7] of string; // will be used for check/update/fill function processing
    AddonsSelected: array of string;    // To save/restore user defined Addon selections
    GnomeAnimations: boolean = true;    // Switch for Gnome Animations
    SetGnomeAnimations: byte = 1;       // Switch for Gnome Animations Program config
    GnomeWaveImgCount: byte = 0;        // Image Counter for Gnome Animation Wave
    GnomeWaveRunTime: integer = 0;      // Runtime Factor to switch-off Animation after n * TimerAniWave.Interval
    GnomeRunImgCount: byte = 0;         // Image Counter for Gnome Animation Run
    CheckedForAddonUpdates: boolean = false; // to enable/disable Update-Button
    AllowSortAddonList: boolean = true; // Switch to disable AddonList-Sorting when using context menu
    SelMark: integer = 0;               // Saves the current selected row (used for context menu)
    FilterSet: boolean = false;         // Switch for Filter-Setting (on/off?)
    InfoDiaAddName: string = '';        // Addon Name for Change overview dialogue
    InfoDiaAddVer: string = '';         // Addon Version for Change overview dialogue
    InfoDiaAddURL: string = '';         // Addon URL for Change overview dialogue
    InfoDiaAddChanges: string = '';     // Addon Changes for Change overview dialogue
    DirectInstall: boolean = true;      // After adding Addon to list, install directly
    SetDirectInstall: byte = 1;         // Program config for above direct install


implementation

{$R *.lfm}

{ TForm1 }

{$include umain_addchkupd.inc}
{$include umain_adddoupd.inc}
{$include umain_addnew.inc}
{$include umain_addrem.inc}
{$include umain_addfill.inc}
{$include umain_listfunc.inc}
{$include umain_filter.inc}
{$include umain_statboxfunc.inc}
{$include umain_formfunc.inc}
{$include umain_menue.inc}
{$include umain_webfunc.inc}
{$include umain_dbupgrade.inc}
{$include umain_anignome.inc}

///////////////////////////////
// FINAL RELEASE VERSION 3.1 //
///////////////////////////////

// TEST BUTTON (not visible in Final versions)

Procedure TForm1.Button1Click(Sender: TObject);
begin
  // FillOutAddonTable; // Function to fill table with random content for testing purposes
  // SortAddonList('Number'); // Sort function accepts 3 parameters: 'Number', 'Name' and 'State'
  // RenumberList // Renumber the index of complete list


end;


end.

